from itertools import combinations, combinations_with_replacement, permutations
from random import randint
from string import ascii_letters, digits

from fakeit.exceptions import EmptyAlphabet, InvalidRange


def fake_literal(alphabet=ascii_letters + digits):
    if not alphabet:
        raise EmptyAlphabet
    return alphabet[randint(0, len(alphabet) - 1)]


def fake_string(
    min_length=0, max_length=16, alphabet=ascii_letters + digits, unique=False
):
    if max_length < min_length:
        raise InvalidRange

    if (max_length > len(alphabet)) and unique:
        raise InvalidRange

    length = randint(min_length, max_length)

    if unique:
        cache = set()
        result = []
        while len(result) != length:
            buf = fake_literal(alphabet=list((set(alphabet) - cache)))
            cache.add(buf)
            result.append(buf)
        return "".join(result)
    else:
        return "".join(fake_literal(alphabet=alphabet) for _ in range(length))


def fake_strings(
    amount,
    min_length=0,
    max_length=16,
    alphabet=ascii_letters + digits,
    unique=False,
    unique_strings=False,
):
    if unique:
        cache = set()
        result = []
        while len(result) != amount:
            buf = fake_string(
                min_length=min_length,
                max_length=max_length,
                alphabet=alphabet,
                unique=unique_strings,
            )
            cache.add(buf)
            result.append(buf)
            yield buf
        else:
            return
    else:
        yield from (
            fake_string(
                min_length=min_length,
                max_length=max_length,
                alphabet=alphabet,
                unique=unique_strings,
            )
            for _ in range(amount)
        )


def all_combinations_fake_string(
    min_length=0, max_length=16, alphabet=ascii_letters + digits
):
    length = randint(min_length, max_length)
    yield from ("".join(x) for x in combinations(alphabet, length))


def all_combinations_with_replacement_fake_string(
    min_length=0, max_length=16, alphabet=ascii_letters + digits
):
    length = randint(min_length, max_length)
    yield from ("".join(x) for x in combinations_with_replacement(alphabet, length))


def all_permutations_fake_string(alphabet=ascii_letters + digits):
    yield from ("".join(x) for x in permutations(alphabet, len(alphabet)))



from hashlib import md5, sha1, sha256, sha384, sha512

from fakeit.basics.bytes import fake_bytes


def fake_md5():
    return md5(fake_bytes()).hexdigest()


def fake_sha1():
    return sha1(fake_bytes()).hexdigest()


def fake_sha256():
    return sha256(fake_bytes()).hexdigest()


def fake_sha384():
    return sha384(fake_bytes()).hexdigest()


def fake_sha512():
    return sha512(fake_bytes()).hexdigest()



from random import randint, uniform


def fake_int(min=0, max=255):
    "from min to max included <[min; max]>"
    return randint(min, max)


def fake_float(min=0, max=1):
    "from min to max included <[min; max]>"
    return uniform(min, max)


def fake_complex(min_real, max_real, min_image, max_image, round=False):
    if round:
        return complex(randint(min_real, max_real), randint(min_image, max_image))
    return complex(uniform(min_real, max_real), uniform(min_image, max_image))



from random import randint


def fake_local(area_code=None, body=None, mediator="-"):
    area_code = area_code or randint(100, 999)
    body = body or randint(1000000, 9999999)
    return f"{area_code}{mediator}{body}"


def fake_domestic(country_code=None, area_code=None, body=None, mediator="-"):
    country_code = country_code or randint(1, 99)
    area_code = area_code or randint(100, 999)
    body = body or randint(1000000, 9999999)
    return f"({country_code}) {area_code}{mediator}{body}"


def fake_dialed(country_code=None, area_code=None, body=None, mediator="-"):
    country_code = country_code or randint(1, 99)
    area_code = area_code or randint(100, 999)
    body = body or randint(1000000, 9999999)
    return f"{country_code}{mediator}{area_code}{mediator}{body}"


def fake_international(country_code=None, area_code=None, body=None, mediator="-"):
    country_code = country_code or randint(1, 99)
    area_code = area_code or randint(100, 999)
    body = body or randint(1000000, 9999999)
    return f"+{country_code}{mediator}{area_code}{mediator}{body}"



from . import emails, names, phones


class Person:
    def __init__(self, first_name, last_name, email, phone):
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.phone = phone

    @property
    def fullname(self):
        return self.first_name + " " + self.last_name

    def __repr__(self):
        return f"<Person {self.fullname}> (FirstName={self.first_name}, LastName={self.last_name}, Email={self.email}, Phone={self.phone})"


def fake_person():
    first_name = names.fake_name()
    last_name = names.fake_surname()
    email = emails.fake_enough_email(addr=f"{first_name}.{last_name}")
    phone = phones.fake_international()
    return Person(first_name=first_name, last_name=last_name, email=email, phone=phone)
